import java.awt.*;
import java.io.*;
import java.util.*;

/**
 * Klasse: MyChars.
 * Hier werden die Informationen ber die einzelnen Zeichen gespeichert & verwaltet.
 */
public class MyChars
    {
    private MagList data;
    Color foreGroundColor, backGroundColor;
    
    /**
     * Klasse: Chars.
     * In diesem Format werden die einzelen Zeichen aus den *.def-Files abgelegt.
     * Ein Eintrag besteht aus der Bezeichnung des Zeichens:
     *   char name
     * und einer einer Liste mit den Anweisungen zum Zeichnen des Zeichens
     *   MagList chardata.
     */
    private class Chars
	{
	char name;
	MagList chardata;
	
	Chars (char s, MagList d)
	    {
	    name = s;
	    chardata = d;
	    }
	}
    
    /**
     * Klasse: LinieKurve
     * In diesem Format werden die einzelnen Anweisungen zum zeichnen von Kurven bzw. Linie gespeichert, wobei
     *	typ == false ... Strich
     *  typ == true .... Kurve
     */
    private class LinieKurve
	{
	boolean typ;
	int abme[];
	
	LinieKurve (boolean t, int a[])
	    {
	    typ = t;
	    abme = a;
	    }
	}
   
    /**
     * Constructor.
     * ld die "*.def" aus dem angegebenen Verzeichnis
     */
    MyChars (String pfad)
	{
	data = new MagList ();
	
	// versucht Directory zu ffnen
	File file = new File (pfad);
		
	// Liet alles Filenamen im angegebenen Directory ein
	String f[] = file.list ();
	
	for (int i = 0; i < f.length; i++)
	    if ((f[i].substring (f[i].length() - 4, f[i].length()).equalsIgnoreCase (".def")) && (f[i].length() == 5))
		readDefFile (pfad, f[i]);
	}

    /**
     * Methode: readDefFile
     * Wertet die im File gegebenen Zeilen aus und Speichert sie in data
     * @param pfad Gibt das Verzeichnis an, in dem sich das File befindet
     * @param file Gibt das zu ladende File an
     */
    private void readDefFile (String pfad, String file)
	{
	String s = null, sk = null;
	MagList lines = new MagList ();
	boolean error = false;

	// ffnet File
	ReadFile deffile = new ReadFile (pfad, file);

	// Liet solange Zeilen aus dem File ein bis das Fileende erreicht ist
	while ((s = deffile.readLine()) != null)
	    {
	    int h[] = new int[5];
	    StringTokenizer t = new StringTokenizer (s, " ");
	    sk = t.nextToken();
	    
	    // Wenn die eingelesene Zeile eine Strichanweisung enthlt
	    if ((t.countTokens () == 4) && (sk.equals ("Strich")))
		{
		for (int i = 0; i < 4; i++)
		    {
		    try {h[i] = Integer.parseInt (t.nextToken ());}
		    catch (NumberFormatException e) {error = true;}
		    if (h[i] > 6) error = true;
		    }
		if ((h[0] > 4) || (h[2] > 4)) error = true;
		lines.add (new LinieKurve (false, h));
		}
	    // Wenn die eingelesene Zeile eine Kurvenanweisung enthlt
	    else if ((t.countTokens () == 5) && (sk.equals ("Kurve")))
		{
		for (int i = 0; i < 5; i++)
		    try {h[i] = Integer.parseInt (t.nextToken ());}
		    catch (NumberFormatException e) {error = true;}
		lines.add (new LinieKurve (true, h));
		if ((h[0] > 4) || (h[1] > 6) || (h[2] > 3)) error = true;
		}
	    // Wenn die eingelesene Zeile keine gltige Anweisung enthlt, Fehlerwert 'true' setzten
	    else error = true;
	    for (int i = 0; i < 5; i++)
		if ((h[i] < 0) || (h[i] > 360)) error = true;
	    }		
	
	// Gibt Fehlermeldung aus, falls beim Einlesen ein Fehler aufgetreten ist
	// Sonst werden die eingelesenen Daten gespeichert
	if (error || (deffile == null)) System.out.println ("Beim Einlesen des Files \"" + file + "\" ist ein Fehler aufgetreten");
	else data.add (new Chars (file.charAt (0), lines));
	}
	    
    
    /**
     * Methode drawMyString.
     * gibt den bergebenen String an der bergebenen XY-Position aus.
     * @param s Der auszugebende String.
     * @param size Der Vergrerungsfaktor.
     * @param x Die Startposition x.
     * @param y Die Startposition y.
     */
    public void drawMyString (Graphics g, String s, int size, int x, int y)
	{
	for (int i = 0; i < s.length (); i++)
	    {
	    findMyChar (g, s.charAt (i), size, x, y);
	    x += size*6;
	    }
	}
    
    /**
     * Methode: findMyChar.
     * Sucht den bergebenen Char in der Liste 'data'.
     * Wird kein Char gefunden, wird das Programm ohne Fehlermeldung fortgesetzt.
     * @param c Das auszugebende Zeichen.
     * @param size Der Vergrerungsfaktor.
     * @param x Die Startposition x.
     * @param y Die Startposition y.
     */
    public void findMyChar (Graphics g, char c, int size, int x, int y)
	{
	for (data.reset (); !data.eol (); data.gotoNext ())
	    if (((Chars)data.getData ()).name == c)
		drawMyChar (g, (((Chars)data.getData()).chardata), size, x, y);
	}
	
    /**
     * Methode: drawMyChar.
     * Geht die Zeichenanweisungen durch, und zeichnet den Char.
     * @param c Liste mit den Zeichenanweisungen.
     * @param size Der Vergrerungsfaktor.
     * @param x Die Startposition x.
     * @param y Die Startposition y.
     */
    private void drawMyChar (Graphics g, MagList c, int size, int x, int y)
	{
	for (c.reset (); !c.eol (); c.gotoNext ())
	    // Wenn true = Kurve, sonst Strich
	    if (((LinieKurve)c.getData()).typ)
		drawKurve (g, ((LinieKurve)c.getData()).abme, size, x, y);
	    else
		drawStrich (g, ((LinieKurve)c.getData()).abme, size, x, y);
	}
    
    /**
     * Methode: drawKurve.
     * Zeichnet einen Bogen. Dabei wird zuerst ein Bogen in der Vordergrundfarbe, und
     * dann ein zweiter um eine Einheit kleinern Bogen und der Hintergrundfarbe gezeichnet.
     * @param d[] Feld mit den Grenangaben fr fillArc().
     * @param size Der Vergrerungsfaktor.
     * @param x Die Startposition x.
     * @param y Die Startposition y.
     */
    private void drawKurve (Graphics g, int d[], int size, int x, int y)
	{
	int r = d[2]*size;
	
	g.setColor(foreGroundColor);
	g.fillArc(x+d[0]*size-r,y+d[1]*size-r,r*2,r*2,d[3],d[4]);
	g.setColor(backGroundColor);
	r -= size;
	g.fillArc(x+d[0]*size-r,y+d[1]*size-r,r*2,r*2,d[3],d[4]);
	g.setColor(foreGroundColor);
       	}
    
    /**
     * Methode: drawStrich.
     * Zeichnet einen Strich, wo bei zwischen Strich nach unten und Strich nach Rechts unterschieden wird.
     * @param d[] Feld mit den Grenangaben fr fillPolygon().
     * @param size Der Vergrerungsfaktor.
     * @param x Die Startposition x.
     * @param y Die Startposition y.
     */
    private void drawStrich (Graphics g, int d[], int size, int x, int y)
	{
	int xpoints [] = {0, 0, 0, 0}, ypoints [] = {0, 0, 0, 0};

	if (d[2]-d[0] > d[3]-d[1])
	    { 
	    // Strich nach Rechts
	    xpoints[0] = xpoints[3] = d[0]*size + x;
	    xpoints[1] = xpoints[2] = (d[2]+1) * size + x;
	    ypoints[0] = d[1]*size + y;
	    ypoints[1] = d[3]*size+ y;
	    ypoints[2] = (d[3]+1)*size + y;
	    ypoints[3] = (d[1]+1)*size + y;
	    }
       else 
	    {
	    // Strich nach Unten
	    xpoints[0] = d[0]*size + x;
	    xpoints[1] = (d[0]+1)*size + x;
	    xpoints[3] = d[2]*size + x;
	    xpoints[2] = (d[2]+1)*size + x;
	    ypoints[0] = ypoints[1] = d[1]*size + y;
	    ypoints[2] = ypoints[3] = (d[3]+1)*size + y;
	    }
	    
	// Zeichnet das Polygon (den Strich) 
	g.fillPolygon (xpoints, ypoints, 4);
	}
    
    /**
     * Methode: setForeBackColor.
     * Setzt die Vorder- und Hintergrundfarbe. Dieser werden von der Methode drawKurve bentigt.
     * Wenn die bergabeparameter null sind, werden die Standardwerte (schwarz/wei) gesetz.
     * @param f Die Vordergrundfarbe.
     * @param b Die Hintergrundfarbe.
     */
    public void setForeBackColor (Color f, Color b)
	{
	if ((foreGroundColor = f) == null) foreGroundColor = Color.black;
	if ((backGroundColor = b) == null) backGroundColor = Color.white;
	}
    }